package com.phantomsxr.armodplugin
import android.app.Activity
import android.graphics.PixelFormat
import android.os.Build
import android.os.Handler
import android.os.Looper
import android.util.Log
import android.view.ViewGroup
import android.view.ViewGroup.LayoutParams.MATCH_PARENT
import android.view.WindowManager
import com.unity3d.player.IUnityPlayerLifecycleEvents
import com.unity3d.player.UnityPlayer
import java.util.concurrent.CopyOnWriteArraySet

class ARMODPlayerUtils {
    companion object {
        private const val LOG_TAG = "ARMODPlayerUtils"

        var activity: Activity? = null
        var options: FlutterARMODWidgetOptions = FlutterARMODWidgetOptions()
        var armodPlayer: UnityPlayer? = null

        var isARMODReady: Boolean = false
        var isARMODPaused: Boolean = false
        var isARMODLoaded: Boolean = false
        var isARMODInBackground: Boolean = false

        val mARMODEventListeners = CopyOnWriteArraySet<ARMODEventListener>()

        /**
         * Create a new ARMOD player with callback
         */
        fun createPlayer(activity: Activity, ule: IUnityPlayerLifecycleEvents, reInitialize: Boolean, callback: OnCreateARMODViewCallback?) {
            if (armodPlayer != null && !reInitialize) {
                callback?.onReady()
                return
            }

            try {
                Handler(Looper.getMainLooper()).post {
                    if (!reInitialize) {
                        activity.window.setFormat(PixelFormat.RGBA_8888)
                        armodPlayer = UnityPlayer(activity, ule)
                    }

                    try {
                        if (!reInitialize) {
                            // wait a moment. fix armod cannot start when startup.
                            Thread.sleep(700)
                        }
                    } catch (e: Exception) {
                    }

                    // start ARMOD
                    if (!reInitialize) {
                        addARMODViewToBackground(activity)
                        armodPlayer!!.windowFocusChanged(true)
                        armodPlayer!!.requestFocus()
                        armodPlayer!!.resume()

                        // restore window layout
                        if (!options.fullscreenEnabled) {
                            activity.window.addFlags(WindowManager.LayoutParams.FLAG_FORCE_NOT_FULLSCREEN)
                            activity.window.clearFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN)
                           // if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.KITKAT) {
                           //     activity.window.addFlags(WindowManager.LayoutParams.FLAG_TRANSLUCENT_STATUS)
                           // }
                        }
                    }

                    isARMODReady = true
                    isARMODLoaded = true

                    callback?.onReady()
                }
            } catch (e: Exception) {
                Log.e(LOG_TAG, e.toString())
            }
        }

        fun postMessage(gameObject: String, methodName: String, message: String) {

            if (!isARMODReady) {
                return
            }

            UnityPlayer.UnitySendMessage(gameObject, methodName, message)
        }

        fun pause() {
            if (armodPlayer != null && isARMODLoaded) {
                armodPlayer!!.pause()
                isARMODPaused = true
            }
        }

        fun resume() {
            if (armodPlayer != null) {
                armodPlayer!!.resume()
                isARMODPaused = false
            }
        }

        fun unload() {
            if (armodPlayer != null) {
                //armodPlayer!!.unload()
                removeARMODViewFromGroup()
                isARMODLoaded = false
            }
        }

        fun moveToBackground() {
            if (armodPlayer != null) {
                isARMODInBackground = true
            }
        }

        fun quitPlayer() {
            try {
                if (armodPlayer != null) {
                    armodPlayer!!.quit()
                    isARMODLoaded = false
                    isARMODReady = false
                }
            } catch (e: Error) {
                e.message?.let { Log.e(LOG_TAG, it) }
            }
        }
        
        fun addARMODEventListener(listener: ARMODEventListener) {
            mARMODEventListeners.add(listener)
        }

        fun removeARMODEventListener(listener: ARMODEventListener) {
            mARMODEventListeners.remove(listener)
        }

        fun addARMODViewToBackground(activity: Activity) {
            if (armodPlayer == null) {
                return
            }
            if (armodPlayer!!.parent != null) {
                (armodPlayer!!.parent as ViewGroup).removeView(armodPlayer)
            }
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                armodPlayer!!.z = -1f
            }
            val layoutParams = ViewGroup.LayoutParams(1, 1)
            activity.addContentView(armodPlayer, layoutParams)
            isARMODInBackground = true
        }

        fun restoreARMODViewFromBackground(activity: Activity) {
            if (armodPlayer == null) {
                return
            }

            if (armodPlayer!!.parent != null) {
                (armodPlayer!!.parent as ViewGroup).addView(armodPlayer)
            }

            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                armodPlayer!!.z = 1f
            }

            val layoutParams = ViewGroup.LayoutParams(1, 1)
            activity.addContentView(armodPlayer, layoutParams)
            isARMODInBackground = false
        }

        fun addARMODViewToGroup(group: ViewGroup) {
            if (armodPlayer == null) {
                return
            }

            if (armodPlayer!!.parent != null) {
                (armodPlayer!!.parent as ViewGroup).removeView(armodPlayer)
            }

            val layoutParams = ViewGroup.LayoutParams(MATCH_PARENT, MATCH_PARENT)
            group.addView(armodPlayer, 0, layoutParams)

            armodPlayer!!.windowFocusChanged(true)
            armodPlayer!!.requestFocus()
            armodPlayer!!.resume()
        }

        fun removeARMODViewFromGroup(){
            if (armodPlayer == null) {
                return
            }

            if (armodPlayer!!.parent != null) {
                (armodPlayer!!.parent as ViewGroup).removeView(armodPlayer)
            }
        }
    }
}