package com.phantomsxr.armodplugin
import android.app.Activity
import android.content.Context
import android.content.ContextWrapper
import android.content.Intent
import android.os.Bundle
import android.os.Handler
import android.os.Looper
import androidx.lifecycle.DefaultLifecycleObserver
import com.unity3d.player.IUnityPlayerLifecycleEvents
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.BinaryMessenger
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.platform.PlatformView
import java.lang.Exception
import android.widget.Toast

class FlutterARMODWidgetController (id: Int,
                                    context: Context,
                                    appContext: Context,
                                    binaryMessenger: BinaryMessenger,
                                    lifecycleProvider: LifecycleProvider,
                                    options: FlutterARMODWidgetOptions
) :     PlatformView,
        DefaultLifecycleObserver,
        ActivityPluginBinding.OnSaveInstanceStateListener,
        FlutterARMODWidgetOptionsSink,
        MethodCallHandler,
        ARMODEventListener,
        IUnityPlayerLifecycleEvents {

    private var lifecycleProvider: LifecycleProvider

    private val methodChannel: MethodChannel
    private val id: Int
    private val context: Context
    private val appContext: Context
    private val options: FlutterARMODWidgetOptions

    private var methodChannelResult: MethodChannel.Result? = null
    private var armodView: ARMODView? = null
    private var disposed: Boolean = false
    val payload: MutableMap<String, Any> = HashMap()
    init {
        // set context and activity
        this.context = context
        this.appContext = appContext

        this.id = id

        // lifecycle
        this.lifecycleProvider = lifecycleProvider

        // set options
        this.options = options

        // setup method channel
        methodChannel = MethodChannel(binaryMessenger, "plugin.phantomsxr.com/armod_view_$id")
        methodChannel.setMethodCallHandler(this)

        // setup armod view
        armodView = getARMODView()


        // Set armod listener
        ARMODPlayerUtils.addARMODEventListener(this)
    }

    fun bootstrap() {
        this.lifecycleProvider.getLifecycle().addObserver(this)
    }

    override fun getView(): ARMODView? {
        return armodView
    }

    private fun getARMODView(): ARMODView? {        
        val view = ARMODView.getInstance(context)
        if (ARMODPlayerUtils.isARMODLoaded) {
            view.player = ARMODPlayerUtils.armodPlayer
        } else {
            createPlayer(view, false)
        }
        return view
    }

    override fun dispose() {
        if (disposed) {
            return
        }
        disposed = true

        //clean all method call,
        // this line in here will be throwUnhandled Exception:
        // MissingPluginException(No implementation found for method [xxx] on channel plugins.)
        Handler().postDelayed({
            methodChannel.setMethodCallHandler(null)
            val lifecycle = lifecycleProvider.getLifecycle()
            if (lifecycle != null) {
                lifecycle.removeObserver(this)
            }
        },500)


    }

    override fun onMethodCall(methodCall: MethodCall, result: MethodChannel.Result) {


        when (methodCall.method) {
            "armod#waitForARMOD" -> {
                if (armodView != null) {
                    result.success(null)
                    return
                }
                methodChannelResult = result
            }
            "armod#createARMOD" -> {
                this.createPlayer(armodView, true)
            }
            "armod#isReady" -> {
                result.success(ARMODPlayerUtils.isARMODReady)
            }
            "armod#isLoaded" -> {
                result.success(ARMODPlayerUtils.isARMODLoaded)
            }
            "armod#isPaused" -> {
                result.success(ARMODPlayerUtils.isARMODPaused)
            }
            "armod#inBackground" -> result.success(ARMODPlayerUtils.isARMODInBackground)
            "armod#postMessage" -> {
                val gameObject: String = methodCall.argument<String>("gameObject").toString()
                val methodName: String = methodCall.argument<String>("methodName").toString()
                val message: String = methodCall.argument<String>("message").toString()

                ARMODPlayerUtils.postMessage(gameObject, methodName, message)
                result.success(true)
            }
            "armod#pausePlayer" -> {
                ARMODPlayerUtils.pause()
                result.success(true)
            }
            "armod#openInNativeProcess" -> {
                openNativeUnity()
                result.success(true)
            }
            "armod#resumePlayer" -> {
                ARMODPlayerUtils.resume()
                result.success(true)
            }
            "armod#unloadPlayer" -> {
                ARMODPlayerUtils.unload()
                result.success(true)
            }
            "armod#dispose" ->                 // TODO: Handle disposing player resource efficiently
                // UnityUtils.unload()
            {
                //ARMODPlayerUtils.unload()
                ARMODPlayerUtils.postMessage("EntryPoint","Dispose","")
                result.success(true)
            }

            "armod#silentQuitPlayer" -> {
                ARMODPlayerUtils.quitPlayer()
                result.success(true)
            }
            "armod#quitPlayer" -> {
                if (ARMODPlayerUtils.armodPlayer != null) {
                    ARMODPlayerUtils.armodPlayer!!.destroy()
                    ARMODPlayerUtils.isARMODLoaded = false
                }
                result.success(true)
            }
            "armod#closeARMOD"->{
                ARMODPlayerUtils.postMessage("EntryPoint","Dispose","")
                //ARMODPlayerUtils.unload()
                //result.success(true)
            }
            "armod#setDevicesOrientation"->{
                val orientation: String = methodCall.argument<String>("orientation").toString()
                ARMODPlayerUtils.postMessage("EntryPoint","SetUIInterfaceOrientation",orientation)
            }
            else -> result.notImplemented()
        }
    }

    override fun onSaveInstanceState(bundle: Bundle) {
        if (disposed) {
            return
        }
    }

    override fun onRestoreInstanceState(bundle: Bundle?) {
        if (disposed) {
            return
        }
    }

    override fun setFullscreenEnabled(fullscreenEnabled: Boolean) {
        ARMODPlayerUtils.options.fullscreenEnabled = fullscreenEnabled
    }


    override fun onUnityPlayerUnloaded() {
        Handler(Looper.getMainLooper()).post {
            methodChannel.invokeMethod("events#onARMODUnloaded", true)
        }
        ARMODPlayerUtils.unload()
    }

    override fun onUnityPlayerQuitted() {
        Handler(Looper.getMainLooper()).post {
            methodChannel.invokeMethod("events#onARMODQuitted", true)
        }
    }

    private fun openNativeUnity() {
        val activity = getActivity(this.context)
        if (activity != null) {
            val intent = Intent(context.applicationContext, ARMODNativeActivity::class.java)
            intent.flags = Intent.FLAG_ACTIVITY_REORDER_TO_FRONT
            intent.putExtra("fullscreen", options.fullscreenEnabled)
            intent.putExtra("flutterActivity", activity.javaClass)
            activity.startActivityForResult(intent, 1)
        }
    }

    private fun createPlayer(view: ARMODView?, reInitialize: Boolean) {
        try {
            val activity = getActivity(context)
            if (activity != null) {
                ARMODPlayerUtils.createPlayer(activity, this, reInitialize, object : OnCreateARMODViewCallback {
                    override fun onReady() {
                        view?.setARMODPlayer(ARMODPlayerUtils.armodPlayer!!)
                        if (methodChannelResult != null) {
                            methodChannelResult!!.success(true)
                            methodChannelResult = null
                        }
                    }
                })
            }
        } catch (e: Exception) {
            if (methodChannelResult != null) {
                methodChannelResult!!.success(false)
                methodChannelResult = null
            }
        }
    }

    fun getActivity(context: Context?): Activity? {
        if (context == null) {
            return null
        } else if (context is ContextWrapper) {
            return if (context is Activity) {
                context
            } else {
                getActivity((context as ContextWrapper).baseContext)
            }
        }
        return null
    }

    override fun onDeviceNotSupport() {
        Handler(Looper.getMainLooper()).post {
            methodChannel.invokeMethod("events#onDeviceNotSupport",null)
        }
    }

    override fun onAddLoadingOverlay() {
        Handler(Looper.getMainLooper()).post {
            methodChannel.invokeMethod("events#onAddLoadingOverlay",null)
        }
    }

    override fun onUpdateLoadingProgress(progress: Float) {
        Handler(Looper.getMainLooper()).post {
            payload["progress"] = progress
            methodChannel.invokeMethod("events#onUpdateLoadingProgress",payload)
        }
    }

    override fun onRemoveLoadingOverlay() {
        Handler(Looper.getMainLooper()).post {
            methodChannel.invokeMethod("events#onRemoveLoadingOverlay",null)
        }
    }

    override fun onThrowException(errorMsg: String, errorCode: Int) {
        Handler(Looper.getMainLooper()).post {
            payload["errorCode"] = errorCode
            payload["errorMsg"] = errorMsg
            methodChannel.invokeMethod("events#onThrowException",payload)
        }
    }

    override fun onNeedInstallARCoreService() {
        Handler(Looper.getMainLooper()).post {
            methodChannel.invokeMethod("events#onNeedInstallARCoreService",null)
        }
    }

    override fun onSdkInitialized() {
        Handler(Looper.getMainLooper()).post {
            methodChannel.invokeMethod("events#onSdkInitialized",null)
        }
    }

    override fun onOpenBuiltInBrowser(url: String) {
        Handler(Looper.getMainLooper()).post {
            payload["url"] = url
            methodChannel.invokeMethod("events#onOpenBuiltInBrowser",payload)
        }
    }

    override fun onRecognitionStart() {
        Handler(Looper.getMainLooper()).post {
            methodChannel.invokeMethod("events#onRecognitionStart",null)
        }
    }

    override fun onRecognitionComplete() {
        Handler(Looper.getMainLooper()).post {
            methodChannel.invokeMethod("events#onRecognitionComplete",null)
        }
    }

    override fun onTryAcquireInformation(opTag: String,androidCallback: AndroidCallback) {
        Handler(Looper.getMainLooper()).post {
            val payload: MutableMap<String, Any> = HashMap()
            payload["opTag"] = opTag
            methodChannel.invokeMethod("events#onTryAcquireInformation",payload, object:MethodChannel.Result{
                override fun notImplemented() {
                    TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
                }

                override fun error(p0:String?,p1:String?,p2: Any?) {
                    print("\n----ERROR------\n")
                    print(p2)
                    print("\n---------------\n")
                }

                override fun success(p0: Any?) {
                    androidCallback.TryAcquireInformationCallback(p0.toString())
                }
            })
        }
    }


    override fun onPackageSizeMoreThanPresetSize(version1: String, version2: String) {
        Handler(Looper.getMainLooper()).post {
            payload["currentSize"] = version1
            payload["presetSize"] = version2
            methodChannel.invokeMethod("events#onPackageSizeMoreThanPresetSize",payload)
        }
    }

    override fun onARMODExit() {
        Handler(Looper.getMainLooper()).post {
            methodChannel.invokeMethod("events#onARMODExit",null)
        }
    }

    override fun onARMODLaunch() {
        Handler(Looper.getMainLooper()).post {
            methodChannel.invokeMethod("events#onARMODLaunch",null)
        }
    }
}