part of flutter_armod_widget;

/// Error thrown when an unknown ARMOD ID is provided to a method channel API.
class UnknownARMODIDError extends Error {
  /// Creates an assertion error with the provided [id] and optional
  /// [message].
  UnknownARMODIDError(this.id, [this.message]);

  /// The unknown ID.
  final int id;

  /// Message describing the assertion error.
  final Object? message;

  String toString() {
    if (message != null) {
      return "Unknown ARMOD ID $id: ${Error.safeToString(message)}";
    }
    return "Unknown ARMOD ID $id";
  }
}

class MethodChannelARMODViewFlutter extends ARMODViewFlutterPlatform {
  /// Every method call passes the int AR-MOD Id
  late final Map<int, MethodChannel> _channels = {};

  /// Accesses the MethodChannel associated to the passed Id.
  MethodChannel channel(int id) {
    MethodChannel? channel = _channels[id];
    if (channel == null) {
      throw UnknownARMODIDError(id);
    }
    return channel;
  }

  MethodChannel ensureChannelInitialized(int id) {
    MethodChannel? channel = _channels[id];
    if (channel == null) {
      channel = MethodChannel('plugin.phantomsxr.com/armod_view_$id');
      channel.setMethodCallHandler(
          (MethodCall call) => _handleMethodCall(call, id));
      _channels[id] = channel;
    }
    return channel;
  }

  /// Initializes the platform interface with [id].
  ///
  /// This method is called when the plugin is first initialized.
  @override
  Future<void> init(int id) {
    MethodChannel channel = ensureChannelInitialized(id);
    return channel.invokeMethod<void>('armod#waitForARMOD');
  }

  /// Dispose of the native resources.
  @override
  Future<void> dispose({int? id}) async {
    // TODO [Rex] will fix this
    // await channel(id!).invokeMethod('armod#dispose');
  }

  // The controller we need to broadcast the different events coming
  // from handleMethodCall.
  //
  // It is a `broadcast` because multiple controllers will connect to
  // different stream views of this Controller.
  final StreamController<ARMODEvent> _armodStreamController =
      StreamController<ARMODEvent>.broadcast();

  /// Returns a filtered view of the events in the _controller, by id.
  Stream<ARMODEvent> _events(int id) =>
      _armodStreamController.stream.where((event) => event.id == id);

  Future<dynamic> _handleMethodCall(MethodCall call, int id) async {
    switch (call.method) {
      case "events#onDeviceNotSupport":
        _armodStreamController.add(deviceNotSupport(id, call.arguments));
        break;
      case "events#onRemoveLoadingOverlay":
        _armodStreamController.add(removeLoadingOverlay(id, call.arguments));
        break;
      case "events#onAddLoadingOverlay":
        _armodStreamController.add(addLoadingOverlay(id, call.arguments));
        break;
      case "events#onUpdateLoadingProgress":
        _armodStreamController.add(updateLoadingProgress(id, call.arguments));
        break;
      case "events#onThrowException":
        _armodStreamController.add(throwException(id, call.arguments));
        break;
      case "events#onNeedInstallARCoreService":
        _armodStreamController
            .add(needInstallARCoreService(id, call.arguments));
        break;
      case "events#onSdkInitialized":
        _armodStreamController.add(sdkInitialized(id, call.arguments));
        break;
      case "events#onOpenBuiltInBrowser":
        _armodStreamController.add(openBuiltInBrowser(id, call.arguments));
        break;
      case "events#onRecognitionStart":
        _armodStreamController.add(recognitionStart(id, call.arguments));
        break;
      case "events#onRecognitionComplete":
        _armodStreamController.add(recognitionComplete(id, call.arguments));
        break;
      case "events#onTryAcquireInformation":
        bool gotResult = false;
        _armodStreamController
            .add(tryAcquireInformation(id, call.arguments, (_result) {
          resultData = _result;
          gotResult = true;
        }));

        //Wait for result
        while (!gotResult) {
          await Future.delayed(Duration(milliseconds: 1));
        }

        return resultData;
      case "events#onPackageSizeMoreThanPresetSize":
        _armodStreamController
            .add(packageSizeMoreThanPresetSize(id, call.arguments));
        break;
      case "events#onARMODExit":
        _armodStreamController.add(onARMODExit(id, call.arguments));
        break;
      case "events#onARMODLaunch":
        _armodStreamController.add(onARMODLaunch(id, call.arguments));
        break;
      default:
        throw UnimplementedError("Unimplemented ${call.method} method");
    }
  }

  ///Is AR-MOD paused
  @override
  Future<bool?> isPaused({required int id}) async {
    return await channel(id).invokeMethod('armod#isPaused');
  }

  ///Is AR-MOD ready
  @override
  Future<bool?> isReady({required int id}) async {
    return await channel(id).invokeMethod('armod#isReady');
  }

  ///Is AR-MOD loaded
  @override
  Future<bool?> isLoaded({required int id}) async {
    return await channel(id).invokeMethod('armod#isLoaded');
  }

  ///Is AR-MOD running in the background
  @override
  Future<bool?> inBackground({required int id}) async {
    return await channel(id).invokeMethod('armod#inBackground');
  }

  ///Create the AR-MOD player
  @override
  Future<bool?> createARMODPlayer({required int id}) async {
    return await channel(id).invokeMethod('armod#createARMOD');
  }

  ///Close and hide AR-MOD Player
  @override
  Future<bool?> closeARMODPlayer({required int id}) async {
    return await channel(id).invokeMethod('armod#closeARMOD');
  }

  @override
  Future<void> setDevicesOrientation(
      {required int id, required String orientation}) async {
    return await channel(id)
        .invokeMethod('armod#setDevicesOrientation', <String, dynamic>{
      'orientation': orientation,
    });
  }

  ///Current device is not support ARMOD
  @override
  Stream<deviceNotSupport> onDeviceNotSupport({required int id}) {
    return _events(id).whereType<deviceNotSupport>();
  }

  ///Remove loading overlay and loading AR experience finished
  @override
  Stream<removeLoadingOverlay> onRemoveLoadingOverlay({required int id}) {
    return _events(id).whereType<removeLoadingOverlay>();
  }

  ///Start loading AR experience
  @override
  Stream<addLoadingOverlay> onAddLoadingOverlay({required int id}) {
    return _events(id).whereType<addLoadingOverlay>();
  }

  ///Loading AR experience asset progress
  ///[progress] loading progress value(0.0~1.0).
  @override
  Stream<updateLoadingProgress> onUpdateLoadingProgress(
      {required int id, double? progress}) {
    return _events(id).whereType<updateLoadingProgress>();
  }

  ///The method will be executed if an exception occurs
  ///[message] error message
  ///[errorCode] the error id
  @override
  Stream<throwException> onThrowException(
      {required int id, String? message, int? errorCode}) {
    return _events(id).whereType<throwException>();
  }

  ///[Android Only]When your device supports ARCore but you are installing the ARCore service,
  ///this method will be executed, prompting the user to install the ARCore service.
  @override
  Stream<needInstallARCoreService> onNeedInstallARCoreService(
      {required int id}) {
    return _events(id).whereType<needInstallARCoreService>();
  }

  ///After the AR algorithm is initialized, the method will be executed
  @override
  Stream<sdkInitialized> onSdkInitialized({required int id}) {
    return _events(id).whereType<sdkInitialized>();
  }

  ///Open the url in builtin browser from AR experience
  @override
  Stream<openBuiltInBrowser> onOpenBuiltInBrowser(
      {required int id, String? url}) {
    return _events(id).whereType<openBuiltInBrowser>();
  }

  ///Start image recognition
  @override
  Stream<recognitionStart> onRecognitionStart({required int id}) {
    return _events(id).whereType<recognitionStart>();
  }

  ///Finished image recognition
  @override
  Stream<recognitionComplete> onRecognitionComplete({required int id}) {
    return _events(id).whereType<recognitionComplete>();
  }

  ///Get any information from app to AR experiences
  @override
  Stream<tryAcquireInformation> onTryAcquireInformation(
      {required int id, String? opTag}) {
    return _events(id).whereType<tryAcquireInformation>();
  }

  ///Detected that the AR experience package is too large
  @override
  Stream<packageSizeMoreThanPresetSize> onPackageSizeMoreThanPresetSize(
      {required int id, String? currentSize, String? presetSize}) {
    return _events(id).whereType<packageSizeMoreThanPresetSize>();
  }

  ///AR-MOD SDK Exited
  @override
  Stream<onARMODExit> onOnARMODExit({required int id}) {
    return _events(id).whereType<onARMODExit>();
  }

  ///AR-MOD SDK Start
  @override
  Stream<onARMODLaunch> onOnARMODLaunch({required int id}) {
    return _events(id).whereType<onARMODLaunch>();
  }

  @override
  Widget buildView(
      Map<String, dynamic> creationParams,
      Set<Factory<OneSequenceGestureRecognizer>>? gestureRecognizers,
      PlatformViewCreatedCallback onPlatformViewCreated,
      bool useAndroidView) {
    final String _viewType = 'plugin.phantomsxr.com/armod_view';

    if (defaultTargetPlatform == TargetPlatform.android) {
      if (useAndroidView) {
        return AndroidView(
          viewType: _viewType,
          onPlatformViewCreated: onPlatformViewCreated,
          gestureRecognizers: gestureRecognizers,
          hitTestBehavior: PlatformViewHitTestBehavior.opaque,
          creationParams: creationParams,
          creationParamsCodec: StandardMessageCodec(),
        );
      }

      return PlatformViewLink(
        viewType: _viewType,
        surfaceFactory:
            (BuildContext context, PlatformViewController controller) {
          return AndroidViewSurface(
            controller: controller as AndroidViewController,
            gestureRecognizers: const <Factory<OneSequenceGestureRecognizer>>{},
            hitTestBehavior: PlatformViewHitTestBehavior.opaque,
          );
        },
        onCreatePlatformView: (PlatformViewCreationParams params) {
          return PlatformViewsService.initSurfaceAndroidView(
            id: params.id,
            viewType: _viewType,
            layoutDirection: TextDirection.ltr,
            creationParams: creationParams,
            creationParamsCodec: StandardMessageCodec(),
          )
            ..addOnPlatformViewCreatedListener(params.onPlatformViewCreated)
            ..addOnPlatformViewCreatedListener(onPlatformViewCreated)
            ..create();
        },
      );
    } else if (defaultTargetPlatform == TargetPlatform.iOS) {
      return UiKitView(
        viewType: _viewType,
        onPlatformViewCreated: onPlatformViewCreated,
        gestureRecognizers: gestureRecognizers,
        creationParams: creationParams,
        creationParamsCodec: const StandardMessageCodec(),
      );
    }
    return Text(
        '$defaultTargetPlatform is not yet supported by the ARMOD player plugin');
  }

  @override
  Future<void> postMessage(
      {required int id,
      required String gameObject,
      required String methodName,
      required String message}) async {
    await channel(id).invokeMethod('armod#postMessage', <String, dynamic>{
      'gameObject': gameObject,
      'methodName': methodName,
      'message': message,
    });
  }

  @override
  Future<void> pausePlayer({required int id}) async {
    await channel(id).invokeMethod('armod#pausePlayer');
  }

  @override
  Future<void> resumePlayer({required int id}) async {
    await channel(id).invokeMethod('armod#resumePlayer');
  }

  @override
  Future<void> openInNativeProcess({required int id}) async {
    await channel(id).invokeMethod('armod#openInNativeProcess');
  }

  @override
  Future<void> unloadPlayer({required int id}) async {
    await channel(id).invokeMethod('armod#unloadPlayer');
  }

  @override
  Future<void> quitPlayer({required int id}) async {
    await channel(id).invokeMethod('armod#quitPlayer');
  }
}
